"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlayerData = void 0;
const protocol_1 = require("@skeldjs/protocol");
const events_1 = require("@skeldjs/events");
const constant_1 = require("@skeldjs/constant");
const objects_1 = require("./objects");
const events_2 = require("./events");
/**
 * Represents the player of a client connected to the room.
 *
 * See {@link PlayerDataEvents} for events to listen to.
 */
class PlayerData extends events_1.EventEmitter {
    constructor(room, 
    /**
     * This player's server-unique client ID.
     */
    clientId, 
    /**
     * The player's login name, not necessarily the display name, see {@link PlayerInfo}.
     */
    username, 
    /**
     * The platform that the player is playing on.
     */
    platform = new protocol_1.PlatformSpecificData(constant_1.Platform.Unknown, "Unknown"), 
    /**
     * The level/rank of the player.
     */
    playerLevel = 0, 
    /**
     * The player's innersloth friend code.
     */
    friendCode = "", 
    /**
     * The player's unique global UUID.
     */
    puid = "") {
        super();
        this.clientId = clientId;
        this.username = username;
        this.platform = platform;
        this.playerLevel = playerLevel;
        this.friendCode = friendCode;
        this.puid = puid;
        this.room = room;
        this.stream = [];
        this.isReady = false;
        this.inScene = false;
        this.control = undefined;
        this.on("component.spawn", () => {
            if (this.hasSpawned) {
                this.emitSync(new events_2.PlayerSpawnEvent(this.room, this));
            }
        });
    }
    async emit(event) {
        this.room.emit(event);
        return super.emit(event);
    }
    async emitSerial(event) {
        this.room.emitSerial(event);
        return super.emitSerial(event);
    }
    emitSync(event) {
        this.room.emitSync(event);
        return super.emitSync(event);
    }
    /**
     * The player's physics component.
     */
    get physics() {
        var _a;
        return (_a = this.control) === null || _a === void 0 ? void 0 : _a.getComponent(objects_1.PlayerPhysics);
    }
    /**
     * The player's movement component.
     */
    get transform() {
        var _a;
        return (_a = this.control) === null || _a === void 0 ? void 0 : _a.getComponent(objects_1.CustomNetworkTransform);
    }
    /**
     * Whether or not this player is a fake player, as in they are entirely
     * client-side and have no real player behind them.
     */
    get isFakePlayer() {
        return this.control && this.clientId === 0;
    }
    /**
     * The player's game information, such as dead/impostor/disconnected states,
     * hats, names, pets, etc.
     */
    get playerInfo() {
        var _a, _b;
        if (this.playerId === undefined) {
            this._playerInfoCachedPlayerId = undefined;
            this._playerInfoCached = undefined;
            this._cacheGameData = undefined;
            return undefined;
        }
        if (this.playerId === this._playerInfoCachedPlayerId && this._playerInfoCached && this.room.gameData === this._cacheGameData) {
            return this._playerInfoCached;
        }
        this._playerInfoCachedPlayerId = this.playerId;
        this._cacheGameData = this.room.gameData;
        this._playerInfoCached = (_b = (_a = this._cacheGameData) === null || _a === void 0 ? void 0 : _a.players) === null || _b === void 0 ? void 0 : _b.get(this.playerId);
        return this._playerInfoCached;
    }
    /**
     * Shorthand for `player.playerInfo.defaultOutfit.name`.
     *
     * This will return the player's name as it appears in-game, not including
     * the name of the player that they might have shapeshifted into.
     */
    get playerName() {
        var _a;
        return (_a = this.playerInfo) === null || _a === void 0 ? void 0 : _a.defaultOutfit.name;
    }
    /**
     * The room-unique player ID of the player.
     */
    get playerId() {
        var _a;
        return (_a = this.control) === null || _a === void 0 ? void 0 : _a.playerId;
    }
    /**
     * Whether or not the player has fully spawned.
     */
    get hasSpawned() {
        return !!(this.control && this.physics && this.transform);
    }
    /**
     * Whether or not the player is the host of the room they belong in.
     */
    get isHost() {
        return this.room.host === this;
    }
    /**
     * Whether or not the player is the current client's player.
     */
    get isMe() {
        return this.room.myPlayer === this;
    }
    /**
     * Mark as readied up to start the game.
     */
    async setReady() {
        this.isReady = true;
        await this.emit(new events_2.PlayerReadyEvent(this.room, this));
        if (this.isMe && !this.isHost) {
            await this.room.broadcast([new protocol_1.ReadyMessage(this.clientId)]);
        }
    }
    /**
     * Despawn all components on the player,
     */
    destroy() {
        if (!this.control)
            return;
        for (const component of this.control.components) {
            component.despawn();
        }
    }
}
exports.PlayerData = PlayerData;
//# sourceMappingURL=PlayerData.js.map